%% uTubeOscV1: dynsysV1 - Function giving the set of dynamic equations
% By Simen Bomnes Valaamo, Hydropower Laboratory NTNU

function df=dynsysV1(t,f) 
%% Constants and variables

global n X Hupper Hlower A TYPE deltaR g rho md numEq Anode 

%% Initialize dfCount and PMAX
c = 1; % Initialize counter, df
turb = 0; % Initialize counter, PMAX
PMAX = zeros(1,n);
dfCount = zeros(n,9);
for i = 1:1:n
    if (X(i,1)==31)
        turb = turb+1;
        dfCount(i,1) = i;
        dfCount(i,2) = c;
        dfCount(i,3) = c+1;
        dfCount(i,4) = c+2;
        c = c+3;
    elseif (X(i,1)==32)
        turb = turb+1;
        dfCount(i,1) = i;
        dfCount(i,2) = c;
        dfCount(i,3) = c+1;
        dfCount(i,4) = c+2;
        dfCount(i,5) = c+3;
        dfCount(i,6) = c+4;
        c = c+5;
    elseif (X(i,1)==33)
        turb = turb+1;
        dfCount(i,1) = i;
        dfCount(i,2) = c;
        dfCount(i,3) = c+1;
        dfCount(i,4) = c+2;
        dfCount(i,5) = c+3;
        dfCount(i,6) = c+4;
        dfCount(i,7) = c+5;
        dfCount(i,8) = c+6;
        PMAX(turb) = X(i,14);
        c = c+7;
    elseif (X(i,1)==34)
        turb = turb+1;
        dfCount(i,1) = i;
        dfCount(i,2) = c;
        dfCount(i,3) = c+1;
        dfCount(i,4) = c+2;
        dfCount(i,5) = c+3;
        dfCount(i,6) = c+4;
        dfCount(i,7) = c+5;
        dfCount(i,8) = c+6;
        dfCount(i,9) = c+7;
        PMAX(turb) = X(i,14);
        c = c+8;
    else
        dfCount(i,1) = i;
        dfCount(i,2) = c;
        c = c+1;
    end
end

%% Change of opening degree
kappa = zeros(1,n);
P = zeros(1,n);
kapparef = zeros(1,n);
fgrid = zeros(1,n);
for m = 1:1:turb
    if (TYPE(m)==1)
        tnr = A(m,1);
        if t < A(m,3)
            kappa(tnr) = A(m,2);
        elseif t < (A(m,3)+A(m,5))
            dkappa = (A(m,2)-A(m,4))/A(m,5);
            kappa(tnr) = A(m,2) - (t-A(m,3))*dkappa;
        elseif t < (A(m,3)+A(m,5)+A(m,7))
            dkappa = (A(m,4)-A(m,6))/A(m,7);
            kappa(tnr) = A(m,4) - (t-A(m,3)-A(m,5))*dkappa;
        else
            kappa(tnr) = A(m,6);
        end

    elseif (TYPE(m)==2)
        tnr = A(m,1);
        if t < A(m,2)
            P(tnr) = PMAX(m);
        elseif t < A(m,4)
            P(tnr) = A(m,3);
        else
            P(tnr) = A(m,5);
        end
        kapparef(tnr) = P(tnr)/PMAX(m);
        
    elseif (TYPE(m)==3)
        tnr = A(m,1);
        if t < A(m,2)
            fgrid(tnr) = X(tnr,15);
        else
            fgrid(tnr) = A(m,3);
        end            
    end
end

%% Set of dynamic eqns.

df = zeros(numEq,1);
for i = 1:1:n
    next = X(i,3);
    prev = X(i,2);
    iEl = dfCount(i,2);
   if (X(i,1)==1) % Element i: Pipe
       if (prev==0) % First element
           if (X(next,1)==31 || X(next,1)==32 ||...
                   X(next,1)==33 || X(next,1)==34) % Next el: turbine
               b = dfCount(next,3);
           else % Next el: not turbine
               b = dfCount(next,2);
           end
           k = X(i,8)*X(i,6)/(2*g*X(i,7)*((X(i,5))^2));
           df(iEl) = (g*X(i,5)/X(i,6))*...
               (Hupper - f(b) - k*f(iEl)*abs(f(iEl))); % dQ/dt          
       elseif (next==0) % Last element
           if (X(prev,1)==31 || X(prev,1)==32 ...
                   || X(prev,1)==33 || X(prev,1)==34) % Prev el: turbine
               a = dfCount(prev,4);
           else % Prev el: not turbine
               a = dfCount(prev,2);
           end
           k = X(i,8)*X(i,6)/(2*g*X(i,7)*((X(i,5))^2));
           df(iEl) = (g*X(i,5)/X(i,6))*(f(a) - ...
               Hlower - k*f(iEl)*abs(f(iEl))); % dQ/dt         
       else 
           if (X(prev,1)==31 || X(prev,1)==32 ||...
                   X(prev,1)==33 || X(prev,1)==34) % Prev el: turbine
               if (X(next,1)==31 || X(next,1)==32 ...
                      || X(next,1)==33 || X(next,1)==34) % Next el: turbine
                   a = dfCount(prev,4);
                   b = dfCount(next,3);
               else % Next el: not turbine
                   a = dfCount(prev,4);
                   b = dfCount(next,2);
               end
           else  % Prev el: not turbine
               if (X(next,1)==31 || X(next,1)==32 || X(next,1)==33 ...
                   || X(next,1)==34) % Next el: turbine
                   a = dfCount(prev,2);
                   b = dfCount(next,3);
               else % Next el: not turbine
                   a = dfCount(prev,2);
                   b = dfCount(next,2);
               end  
           end
           k = X(i,8)*X(i,6)/(2*g*X(i,7)*((X(i,5))^2));
           df(iEl) = (g*X(i,5)/X(i,6))*(f(a) - f(b) -...
               k*f(iEl)*abs(f(iEl))); % dQ/dt         
       end
   elseif (X(i,1)==2) % Element i: S.shaft
       a = dfCount(prev,2);
       b = dfCount(next,2);
       df(iEl) = (f(a)-f(b))/X(i,5);
   elseif (X(i,1)==4) % Element i: Pipe-pipe connection
       a = dfCount(prev,2);
       b = dfCount(next,2);
       df(iEl) = (f(a)-f(b))/Anode;
   elseif (X(i,1)==5) % Element i: Branch
       prevBranch = X(i,4);
       aBranch = dfCount(prevBranch,2);
       nOut = X(i,3);
       fOut = zeros(1,nOut);
       for k = 1:1:nOut
           temp = X(i,4+k);
           btemp = dfCount(temp,2);
           fOut(k) = f(btemp);
       end
       sumfOut = sum(fOut);
       df(iEl) = (f(aBranch)-sumfOut)/Anode;
   elseif (X(i,1)==6) % Element i: Bound
       nextBound = X(i,4);
       bBound = dfCount(nextBound,2);
       nIn = X(i,3);
       fIn = zeros(1,nIn);
       for k = 1:1:nIn
           temp = X(i,4+k);
           atemp = dfCount(temp,2);
           fIn(k) = f(atemp);
       end
       sumfIn = sum(fIn);
       df(iEl) = (sumfIn-f(bBound))/Anode;
   elseif (X(i,1)==31) % Element i: Pelton turbine 
       a = dfCount(prev,2);
       b = dfCount(next,2);
       before = dfCount(i,3);
       after = dfCount(i,4);       
       Ht = X(i,6)*((f(iEl)./(kappa(i)*X(i,7)))^2); % Pressure over turbine
       df(iEl) = (1/X(i,5))*(f(before) - f(after) - Ht); % dQ/dt
       df(before) = (f(a)-f(iEl))/Anode; % Node before
       df(after) = (f(iEl)-f(b))/Anode; % Node after
   elseif (X(i,1)==32) % Element i: Francis turbine, grid mode 
       a = dfCount(prev,2);
       b = dfCount(next,2);
       before = dfCount(i,3);
       after = dfCount(i,4);
       Omega = dfCount(i,5);
       Delta = dfCount(i,6);
       wgrid = 2*pi*X(i,15);
       s = (X(i,8)^2)*(1 - (X(i,9)^2)/(X(i,8)^2))/8; % Turbine geometry
       rA1 = 1/(2*pi*X(i,10)); % rt1/At1
       rA2 = 2/(pi*X(i,9)); % rt2/At2
       alpha = abs(asind(kappa(i)*sind(X(i,11)))); % Guide vane angle
       TgR = 1.2*(rho*X(i,7)*(rA1*X(i,7)*cotd(X(i,11)) ...
           + rA2*X(i,7)*cotd(X(i,12)) -...
           (X(i,9)^2)*X(i,17)/4)); % Rated generator torque
       Tg = TgR*sind(f(Delta))/sind(deltaR); % Generator torque
       Ht = X(i,6)*((f(iEl)./(kappa(i)*X(i,7)))^2) + s*(f(Omega)^2 ...
       - X(i,17)^2)/g; % Pressure over turbine
       df(iEl) = (1/X(i,5))*(f(before) - f(after) - Ht); % dQ/dt
       df(before) = (f(a)-f(iEl))/Anode; % Node before
       df(after) = (f(iEl)-f(b))/Anode; % Node after
       df(Omega) = (X(i,17)^2)/(X(i,14)*(10^6)*X(i,16))*...
           (rho*f(iEl)*(rA1*f(iEl)*cotd(alpha) +...
           rA2*f(iEl)*cotd(X(i,12)) - (X(i,9)^2)*f(Omega)/4)...
           - Tg - md*(X(i,13)*f(Omega) - wgrid)); % d(omega)/dt
       df(Delta) = X(i,13)*f(Omega) - wgrid; % d(delta)/dt
       
   elseif (X(i,1)==33) % Element i: Francis turbine w/freq.governor, grid mode 
       a = dfCount(prev,2);
       b = dfCount(next,2);
       before = dfCount(i,3);
       after = dfCount(i,4);
       Omega = dfCount(i,5);
       Delta = dfCount(i,6);
       KAPPA = dfCount(i,7);
       Servo = dfCount(i,8);
       wgrid = 2*pi*fgrid(i);
       s = (X(i,8)^2)*(1 - (X(i,9)^2)/(X(i,8)^2))/8; % Turbine geometry
       rA1 = 1/(2*pi*X(i,10)); % rt1/At1
       rA2 = 2/(pi*X(i,9)); % rt2/At2
       alpha = abs(asind(f(KAPPA)*sind(X(i,11)))); % Guide vane angle
       TgR = 1.2*(rho*X(i,7)*(rA1*X(i,7)*cotd(X(i,11)) + rA2*X(i,7)...
           *cotd(X(i,12)) - (X(i,9)^2)*X(i,17)/4)); % Rated generator torque
       Tg = TgR*sind(f(Delta))/sind(deltaR); % Generator torque
       Ht = X(i,6)*((f(iEl)./(f(KAPPA)*X(i,7)))^2) + s*(f(Omega)^2 ...
       - X(i,17)^2)/g; % Pressure over turbine
       df(iEl) = (1/X(i,5))*(f(before) - f(after) - Ht); % dQ/dt
       df(before) = (f(a)-f(iEl))/Anode; % Node before
       df(after) = (f(iEl)-f(b))/Anode; % Node after
       df(Omega) = (X(i,17)^2)/(X(i,14)*(10^6)*X(i,16))*...
           (rho*f(iEl)*(rA1*f(iEl)*cotd(alpha) +...
           rA2*f(iEl)*cotd(X(i,12)) - (X(i,9)^2)*f(Omega)/4)...
           - Tg - md*(X(i,13)*f(Omega) - wgrid)); % d(omega)/dt
       df(Delta) = X(i,13)*f(Omega) - wgrid; % d(delta)/dt
       df(KAPPA) = f(Servo); % d(kappa)/dt
       df(Servo) = (1/X(i,18))*(-(1/X(i,20)/X(i,17))*((X(i,17)^2)...
           /(X(i,14)*(10^6)*X(i,16))*...
           (rho*f(iEl)*(rA1*f(iEl)*cotd(alpha) +...
           rA2*f(iEl)*cotd(X(i,12)) - (X(i,9)^2)*f(Omega)/4) - ...
           Tg - md*(X(i,13)*f(Omega) - wgrid))) +...
           (1/X(i,20)/X(i,19))*((X(i,17)-f(Omega))/X(i,17)) -...
           (X(i,21)*X(i,18)+X(i,20)*X(i,19))/...
           (X(i,20)*X(i,19))*f(Servo) -...
           X(i,21)/(X(i,20)*X(i,19))*(1-f(KAPPA))); % dc/dt
       
   elseif (X(i,1)==34) % Element i: Francis turbine w/freq.governor, island mode 
       a = dfCount(prev,2);
       b = dfCount(next,2);
       before = dfCount(i,3);
       after = dfCount(i,4);
       Omega = dfCount(i,5);
       Delta = dfCount(i,6);
       KAPPA = dfCount(i,7);
       Servo = dfCount(i,8);
       Volt = dfCount(i,9);
       wgrid = 2*pi*X(i,15); % Angular grid frequency
       s = (X(i,8)^2)*(1 - (X(i,9)^2)/(X(i,8)^2))/8; % Turbine geometry
       rA1 = 1/(2*pi*X(i,10)); % rt1/At1
       rA2 = 2/(pi*X(i,9)); % rt2/At2
       kphi = X(i,22)/wgrid; % Magnetic flux 
       T0 = X(i,14)*(10^6)/X(i,17); % Start torque
       phi = 0; % Phase angle 
       Rgrid = X(i,22)*kphi*cos(phi)/T0; % Ohmic resistance on the grid
       alpha = abs(asind(f(KAPPA)*sind(X(i,11)))); % Guide vane angle
       TgR = 1.2*(rho*X(i,7)*(rA1*X(i,7)*cotd(X(i,11)) + ...
           rA2*X(i,7)*cotd(X(i,12)) - ...
           (X(i,9)^2)*X(i,17)/4)); % Rated generator torque
       Tg = TgR*sind(f(Delta))/sind(deltaR); % Generator torque
       Ht = X(i,6)*((f(iEl)./(f(KAPPA)*X(i,7)))^2) + s*(f(Omega)^2 ...
       - X(i,17)^2)/g; % Pressure over turbine
       df(iEl) = (1/X(i,5))*(f(before) - f(after) - Ht); % dQ/dt
       df(before) = (f(a)-f(iEl))/Anode; % Node before
       df(after) = (f(iEl)-f(b))/Anode; % Node after
       df(Omega) = (X(i,17)^2)/(X(i,14)*(10^6)*X(i,16))*...
           (rho*f(iEl)*(rA1*f(iEl)*cotd(alpha) +...
           rA2*f(iEl)*cotd(X(i,12)) - (X(i,9)^2)*f(Omega)/4) - Tg...
           - md*(X(i,13)*f(Omega) - f(Volt)/kphi)); % d(omega)/dt
       df(Delta) = X(i,13)*f(Omega) - f(Volt)/kphi; % d(delta)/dt
       df(KAPPA) = f(Servo); % d(kappa)/dt
       df(Servo) = (kapparef(i)/X(i,18))*(-(1/X(i,20)/X(i,17))*...
           ((X(i,17)^2)/(X(i,14)*(10^6)*X(i,16))*...
           (rho*f(iEl)*(rA1*f(iEl)*cotd(alpha) +...
           rA2*f(iEl)*cotd(X(i,12)) - (X(i,9)^2)*f(Omega)/4)...
           - Tg - md*(X(i,13)*f(Omega) - f(Volt)/kphi))) +...
           (1/X(i,20)/X(i,19))*((X(i,17)-f(Omega))/X(i,17))...
           - (X(i,21)*X(i,18)+X(i,20)*X(i,19))/...
           (X(i,20)*X(i,19))*f(Servo) -...
           X(i,21)/(X(i,20)*X(i,19))*(kapparef(i)-f(KAPPA))); % dc/dt
       df(Volt) = Rgrid*Tg/(kphi*cos(phi)) - f(Volt); % dE/dt
       
   end
end
end