%% uTubeOscV1 - Transient modeling of hydropower plants and simulation of U-tube oscillations
% By Simen Bomnes Valaamo, Hydropower Laboratory NTNU

%% Constants 
global n X Hupper Hlower deltaR A g rho md TYPE numEq Anode 

g = 9.81;
rho = 1000;
md = 0.01;
deltaR = 15;
deltaStart = 12.45557;

%% Input from user

% Textfile
prompt1 = {'Enter text file(textfile.txt):'};
name1 = 'Text file';
ans1 = inputdlg(prompt1,name1);
textfile = char(ans1);

% Water level at reservoirs
promptH = {'Upper reservoir, H_upper:','Lower reservoir, H_lower:'};
nameH = 'Head at reservoirs';
ansH = inputdlg(promptH,nameH);
tempH = char(ansH);
ANSH = str2num(tempH);

Hupper = ANSH(1,1);
Hlower = ANSH(2,1);

% Cross sectional area of VSSs
promptNode = {'VSS area(Between 0.1 and 0.001):'};
nameNode = 'VSS area';
ansNode = inputdlg(promptNode,nameNode);
tempNode = char(ansNode);
ANSNode = str2num(tempNode);

Anode = ANSNode(1,1);

% Number of turbines
promptNT = {'Number of turbines in the system:'};
nameNT = 'Turbines';
ansNT = inputdlg(promptNT,nameNT);
tempNT = char(ansNT);
ANSNT = str2num(tempNT);
A = zeros(ANSNT,7);
TYPE = zeros(1,ANSNT);

% Turbine type
for iter = 1:1:ANSNT
    choice1 = questdlg('What type of turbine is modeled?',...
        'Turbine type','Pelton','Francis','Pelton');
    switch choice1
        case 'Pelton'
            prompt2 = {'Element nr. for the turbine:',...
                'Starting value of kappa, K_start:',...
                'Time for start of change[s}:',...
                'First new value of kappa, K_mid:',...
            'Time from K_start to K_mid[s]:', ...
            'Ending value of kappa, K_end:', ...
            'Time from K_mid to K_end[s]:'};
            name2 = 'Manual change of kappa';
            answer = inputdlg(prompt2,name2);
            temp = char(answer);
            a = str2num(temp);
            A(iter,:) = a;
            TYPE(iter) = 1; % kappa manual
        case 'Francis'
            choice2 = questdlg('What type of turbine model?',...
                'Francis turbine',...
                'Simple-Grid(32)','W/Freq.gov-Grid(33)',...
                'W/Freq.gov-Island(34)','Simple-Grid(32)');
            switch choice2
                case 'Simple-Grid(32)'
                    prompt2 = {'Element nr. for the turbine:',...
                        'Starting value of kappa, K_start:',...
                        'Time for start of change[s]:',...
                        'First new value of kappa, K_mid:',...
                        'Time from K_start to K_mid[s]:', ...
                        'Ending value of kappa, K_end:', ...
                        'Time from K_mid to K_end[s]:'};
                    name2 = 'Manual change of kappa';
                    answer = inputdlg(prompt2,name2);
                    temp = char(answer);
                    a = str2num(temp);
                    A(iter,:) = a;
                    TYPE(iter) = 1; % kappa manual
                case 'W/Freq.gov-Grid(33)'
                    prompt2 = {'Element nr. for the turbine:',...
                        'Time for start of change[s]:',...
                        'New grid frequency[Hz]:'};
                    name2 = 'Change of grid frequency';
                    answer = inputdlg(prompt2,name2);
                    temp = char(answer);
                    a = str2num(temp);
                    A(iter,1:3) = a;
                    TYPE(iter) = 3; % grid frequency
                case 'W/Freq.gov-Island(34)'
                    prompt = msgbox('The model is not in use',...
                        'Error','error');
            end
    end
end

% Simulation time
promptTime = {'Total simulation time, tmax:'};
nameTime = 'Simulation time';
TIME = inputdlg(promptTime,nameTime);
tempTime = char(TIME);
tmax = str2num(tempTime);

%% Import data from text file

fID = fopen(textfile);
C = textscan(fID, '%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f',...
             'Delimiter',{',',';',' '}, 'CommentStyle', '%',...
             'TreatAsEmpty',{'NaN'}, 'CollectOutput',1);                                
fclose(fID);
   
Y = cell2mat(C);
[totR,totC] = size(Y);

X = zeros(totR,totC);

for i = 1:totR
    X(i,:) = Y(i,:);
end

[n,m] = size(X);

%% Decide the total number of eqns.

Extra = 0; % Counter, additional eqns
for i = 1:1:n
    if (X(i,1)==31)
        Extra = Extra + 2;
    elseif (X(i,1)==32)
        Extra = Extra + 4;
    elseif (X(i,1)==33)
        Extra = Extra + 6;
    elseif (X(i,1)==34)
        Extra = Extra + 7;
    end
end

numEq = n + Extra; % Total number of eqns

%% ode45 solver 

% Initial values for dynamic eqns
figVal = zeros(n,2);
f0 = zeros(1,numEq);
j = 1;
for i = 1:1:n
    if (X(i,1)==1) % Pipe
        f0(j) = X(i,4);
        j = j+1;
    elseif (X(i,1)==2) % S.shaft
        figVal(i,1) = i;
        figVal(i,2) = j;
        f0(j) = X(i,4);
        j = j+1;
    elseif (X(i,1)==4) % Pipe-pipe connection
        f0(j) = X(i,4);
        j = j+1;
    elseif (X(i,1)==5) % Branch
        f0(j) = X(i,2);
        j = j+1;
    elseif (X(i,1)==6) % Bound
        f0(j) = X(i,2);
        j = j+1;
    elseif (X(i,1)==31) % Pelton turbine
        f0(j) = X(i,4);
        f0(j+1) = X(i,8);
        f0(j+2) = X(i,9);
        j = j+3;
    elseif (X(i,1)==32) % Francis turbine, grid mode
        qt = 1;
        for q = 1:1:ANSNT
            if (A(qt,1)==i)
                deltaConstant = A(qt,2);
                break
            else
                qt = qt+1;
            end
        end       
        f0(j) = X(i,4);
        f0(j+1) = X(i,18);
        f0(j+2) = X(i,19);
        f0(j+3) = X(i,17); 
        f0(j+4) = deltaConstant*deltaStart;
        j = j+5;
    elseif (X(i,1)==33) % Francis turbine w/freq.governor, grid mode
        f0(j) = X(i,4);
        f0(j+1) = X(i,22);
        f0(j+2) = X(i,23);
        f0(j+3) = X(i,17);
        f0(j+4) = deltaStart;
        f0(j+5) = 1;
        f0(j+6) = 0;
        j = j+7;
    elseif (X(i,1)==34) % Francis turbine w/freq.governor, island mode
        f0(j) = X(i,4);
        f0(j+1) = X(i,23);
        f0(j+2) = X(i,24);
        f0(j+3) = X(i,17);
        f0(j+4) = deltaStart;
        f0(j+5) = 1;
        f0(j+6) = 0;
        f0(j+7) = X(i,22);
        j = j+8;
    end
end

% Solver for the set of dynamic eqns, Runge-Kutta-Fehlberg

tspan = [0 tmax];
[T,f] = ode45(@dynsysV1,tspan,f0);

%% Plots

figN = 0;

for p=1:1:n
    
    if(X(p,1)==2)
        tempF = figVal(p,2);
        figN = figN + 1;
        figure(figN)
        title(['Surge shaft: Element ' num2str(p)])
        axis auto
        xlabel('Time [s]')
        ylabel('Water level [m]')
        grid
        hold on
        plot(T,f(:,tempF))
    end
end
